from sage.all_cmdline import *; 
def example_0():	r"""
>>> E = EllipticCurve('11a'); E###line 655:_sage_    : E = EllipticCurve('11a'); E
Elliptic Curve defined by y^2 + y = x^3 - x^2 - 10*x - 20
over Rational Field
>>> E.rank()###line 658:_sage_    : E.rank()
0


\noindent{}Next we compute the number $L(E,1)$ to double precision (as an element
of the real double field {\tt RDF}):

>>> L = RDF(E.Lseries(Integer(1))); L###line 666:_sage_    : L = RDF(E.Lseries(1)); L
0.253841860856


\noindent{}We next compute the real period:

>>> Om = RDF(E.omega()); Om###line 673:_sage_    : Om = RDF(E.omega()); Om
1.26920930428


\noindent{}To compute $\prod c_p$ we factor
the discriminant of $E$.  It turns at that only $11$
divides the discriminant, and since the reduction
at $11$ is split multiplicative the Tamagawa number
is $5=\ord_{11}(\Delta_E)$.

>>> factor(discriminant(E))###line 684:_sage_    : factor(discriminant(E))
-1 * 11^5
>>> c11 = E.tamagawa_number(Integer(11)); c11###line 686:_sage_    : c11 = E.tamagawa_number(11); c11
5


\noindent{}Next we compute the regulator, which is $1$ since $E$
rank $0$.

>>> Reg = RDF(E.regulator()); Reg###line 694:_sage_    : Reg = RDF(E.regulator()); Reg
1.0


\noindent{}The torsion subgroup has order $5$.

>>> T = E.torsion_order(); T###line 701:_sage_    : T = E.torsion_order(); T
5


\noindent{}Putting everything together in \eqref{eq:bsd} and
solving for the conjectural order of $\Sha(E/\QQ)$, we
see that Conjecture~\ref{conj:bsdf} for $E$ is equivalent
to the assertion that $\Sha(E/\QQ)$ has order $1$.

>>> Sha_conj = L * T**Integer(2) / (Om * Reg * c11); Sha_conj###line 711:_sage_    : Sha_conj = L * T^2 / (Om * Reg * c11); Sha_conj
1.0
"""

def example_1():	r"""
>>> E = EllipticCurve('681b'); E###line 720:_sage_    : E = EllipticCurve('681b'); E
Elliptic Curve defined by y^2 + x*y  = x^3 + x^2 - 1154*x - 15345
over Rational Field
>>> E.rank()###line 723:_sage_    : E.rank()
0
>>> L = RDF(E.Lseries(Integer(1))); L###line 725:_sage_    : L = RDF(E.Lseries(1)); L
1.84481520613
>>> Om = RDF(E.omega()); Om###line 727:_sage_    : Om = RDF(E.omega()); Om
0.81991786939


\noindent{}There are two primes of bad reduction this time.

>>> factor(Integer(681))###line 734:_sage_    : factor(681)
3 * 227
>>> factor(discriminant(E))###line 736:_sage_    : factor(discriminant(E))
3^10 * 227^2
>>> c3 = E.tamagawa_number(Integer(3)); c227 = E.tamagawa_number(Integer(227))###line 738:_sage_    : c3 = E.tamagawa_number(3); c227 = E.tamagawa_number(227)
>>> c3, c227###line 739:_sage_    : c3, c227
(2, 2)
>>> Reg = RDF(E.regulator()); Reg###line 741:_sage_    : Reg = RDF(E.regulator()); Reg
1.0
>>> T = E.torsion_order(); T###line 743:_sage_    : T = E.torsion_order(); T
4


\noindent{}In this case it turns out that
$\#\Sha(E/\QQ)$ is conjecturally $9$.

>>> Sha_conj = L * T**Integer(2) / (Om * Reg * c3*c227); Sha_conj###line 751:_sage_    : Sha_conj = L * T^2 / (Om * Reg * c3*c227); Sha_conj
9.0
"""

def example_2():	r"""
>>> E = EllipticCurve('37a'); E###line 761:_sage_    : E = EllipticCurve('37a'); E
Elliptic Curve defined by y^2 + y = x^3 - x over
Rational Field
>>> E.rank()###line 764:_sage_    : E.rank()
1


\par\noindent{}We next compute the value $L'(E,1)$. The
corresponding function in \sage takes a bound on the number
of terms of the $L$-series to use, and returns an approximate
to $L'(E,1)$ along with a bound on the error (coming from
the tail end of the series).

>>> L, error = E.Lseries_deriv_at1(Integer(200)); L, error###line 775:_sage_    : L, error = E.Lseries_deriv_at1(200); L, error
(0.305999773834879, 2.10219814818300e-90)
>>> L = RDF(L); L###line 777:_sage_    : L = RDF(L); L
0.305999773835


\par\noindent{}We compute $\Omega_E$ and the Tamagawa number,
regulator, and torsion as above.

>>> Om = RDF(E.omega()); Om###line 785:_sage_    : Om = RDF(E.omega()); Om
5.98691729246
>>> factor(discriminant(E))###line 787:_sage_    : factor(discriminant(E))
37
>>> c37 = Integer(1)###line 789:_sage_    : c37 = 1
>>> Reg = RDF(E.regulator()); Reg###line 790:_sage_    : Reg = RDF(E.regulator()); Reg
0.05111140824
>>> T = E.torsion_order(); T###line 792:_sage_    : T = E.torsion_order(); T
1


\par\noindent{}Finally, we solve and find that
the conjectural order of $\Sha(E/\Q)$ is $1$.

>>> Sha_conj = L * T**Integer(2) / (Om * Reg * c37); Sha_conj###line 800:_sage_    : Sha_conj = L * T^2 / (Om * Reg * c37); Sha_conj
1.0
"""

def example_3():	r"""
>>> E = EllipticCurve('389a'); E###line 808:_sage_    : E = EllipticCurve('389a'); E
Elliptic Curve defined by y^2 + y = x^3 + x^2 - 2*x
over Rational Field
>>> E.rank()###line 811:_sage_    : E.rank()
2


\noindent{}Because the curve has rank $2$, we use Dokchitser's
$L$-function package to approximate $L^{(2)}(E,1)$
to high precision:

>>> Lser = E.Lseries_dokchitser()###line 820:_sage_    : Lser = E.Lseries_dokchitser()
>>> L = RDF(abs(Lser.derivative(Integer(1),Integer(2)))); L###line 821:_sage_    : L = RDF(abs(Lser.derivative(1,2))); L
1.51863300058


\noindent{}We compute the regulator, Tamagawa numbers, and torsion
as usual:

>>> Om = RDF(E.omega()); Om###line 829:_sage_    : Om = RDF(E.omega()); Om
4.98042512171
>>> factor(discriminant(E))###line 831:_sage_    : factor(discriminant(E))
389
>>> c389 = Integer(1)###line 833:_sage_    : c389 = 1
>>> Reg = RDF(E.regulator()); Reg###line 834:_sage_    : Reg = RDF(E.regulator()); Reg
0.152460177943
>>> T = E.torsion_order(); T###line 836:_sage_    : T = E.torsion_order(); T
1


Finally we solve for the conjectural order of $\#\Sha(E/\QQ)$.

>>> Sha_conj = (L/Integer(2)) * T**Integer(2) / (Om * Reg * c389)###line 843:_sage_    : Sha_conj = (L/2) * T^2 / (Om * Reg * c389)
>>> Sha_conj###line 844:_sage_    : Sha_conj
1.0
"""

def example_4():	r"""
>>> E = EllipticCurve('5077a'); E###line 872:_sage_    : E = EllipticCurve('5077a'); E
Elliptic Curve defined by y^2 + y = x^3 - 7*x + 6
over Rational Field
>>> E.rank()###line 875:_sage_    : E.rank()
3


\noindent{}We compute $L(E,s)$ using Dokchitser's algorithm.  Note
that the order of vanishing appears to be $3$.

>>> E.root_number()###line 883:_sage_    : E.root_number()
-1
>>> Lser = E.Lseries_dokchitser()###line 885:_sage_    : Lser = E.Lseries_dokchitser()
>>> Lser.derivative(Integer(1),Integer(1))###line 886:_sage_    : Lser.derivative(1,1)
-5.63436295355925e-22
>>> Lser.derivative(Integer(1),Integer(2))###line 888:_sage_    : Lser.derivative(1,2)
2.08600476044634e-21
>>> L = RDF(abs(Lser.derivative(Integer(1),Integer(3)))); L###line 890:_sage_    : L = RDF(abs(Lser.derivative(1,3))); L
10.3910994007


\noindent{}That the order of vanishing is really $3$ follows from the
Gross-Zagier theorem, which asserts that $L'(E,1)$ is a nonzero
multiple of the N\'eron-Tate canonical height of a certain point on
$E$ called a Heegner point.  One can explicitly construct this
point\footnote{This is not yet implemented in \sage; if it were, there
  would be an example right here.} on $E$ and find that it is torsion,
hence has height $0$, so $L'(E,1)=0$. That $L''(E,1)=0$ then follows
from the functional equation (see Section~\ref{sec:complexles}).
Finally we compute the other BSD invariants:

>>> Om = RDF(E.omega()); Om###line 905:_sage_    : Om = RDF(E.omega()); Om
4.15168798309
>>> factor(discriminant(E))###line 907:_sage_    : factor(discriminant(E))
5077
>>> c5077 = Integer(1)###line 909:_sage_    : c5077 = 1
>>> Reg = RDF(E.regulator()); Reg###line 910:_sage_    : Reg = RDF(E.regulator()); Reg
0.417143558758
>>> T = E.torsion_order(); T###line 912:_sage_    : T = E.torsion_order(); T
1


\noindent{}Putting everything together we see that the
conjectural order of $\Sha(E/\QQ)$ is $1$.

>>> Sha_conj = (L/Integer(6)) * T**Integer(2) / (Om * Reg * c5077)###line 920:_sage_    : Sha_conj = (L/6) * T^2 / (Om * Reg * c5077)
>>> Sha_conj###line 921:_sage_    : Sha_conj
1.0
"""

def example_5():	r"""
>>> E = EllipticCurve([Integer(1), -Integer(1), Integer(0), -Integer(79), Integer(289)]); E###line 938:_sage_    : E = EllipticCurve([1, -1, 0, -79, 289]); E
Elliptic Curve defined by y^2 + x*y  = x^3 - x^2 - 79*x + 289
over Rational Field
>>> E.rank()###line 941:_sage_    : E.rank()
4


We next compute $L(E,1)$, $L'(E,1)$, $L^{(2)}(E,1)$, $L^{(3)}(E,1)$,
and $L^{(4)}(E,1)$.  All these special values {\em look} like they are
$0$, except for $L^{(4)}(E,1)$ which is about $214$, hence clearly
nonzero.  One can prove that $L(E,1)=0$ (e.g., using denominator
bounds coming from modular symbols), hence since the root number is
$+1$, we have either $r_{E,\an} = 2$ or $r_{E,\an}=4$, and of course
suspect (but cannot prove yet) that $r_{E,\an}=4$.
\noindent{}

>>> E.root_number()###line 955:_sage_    : E.root_number()
1
>>> Lser = E.Lseries_dokchitser()###line 957:_sage_    : Lser = E.Lseries_dokchitser()
>>> Lser(Integer(1))###line 958:_sage_    : Lser(1)
1.43930352980778e-18
>>> Lser.derivative(Integer(1),Integer(1))###line 960:_sage_    : Lser.derivative(1,1)
-4.59277879927938e-24
>>> Lser.derivative(Integer(1),Integer(2))###line 962:_sage_    : Lser.derivative(1,2)
-8.85707917856308e-22
>>> Lser.derivative(Integer(1),Integer(3))###line 964:_sage_    : Lser.derivative(1,3)
1.01437455701212e-20
>>> L = RDF(abs(Lser.derivative(Integer(1),Integer(4)))); L###line 966:_sage_    : L = RDF(abs(Lser.derivative(1,4))); L
214.652337502


\noindent{}As above, we compute the other BSD invariants of $E$.

>>> Om = RDF(E.omega()); Om###line 973:_sage_    : Om = RDF(E.omega()); Om
2.97267184726
>>> factor(discriminant(E))###line 975:_sage_    : factor(discriminant(E))
2^2 * 117223
>>> c2 = Integer(2)###line 977:_sage_    : c2 = 2
>>> c117223 = Integer(1)###line 978:_sage_    : c117223 = 1
>>> Reg = RDF(E.regulator()); Reg###line 979:_sage_    : Reg = RDF(E.regulator()); Reg
1.50434488828
>>> T = E.torsion_order(); T###line 981:_sage_    : T = E.torsion_order(); T
1


Finally, putting everything together, we see that the conjectural
order of $\Sha(E/\QQ)$ is 1.
\noindent{}

>>> Sha_conj = (L/Integer(24)) * T**Integer(2) / (Om * Reg * c2 * c117223)###line 990:_sage_    : Sha_conj = (L/24) * T^2 / (Om * Reg * c2 * c117223)
>>> Sha_conj###line 991:_sage_    : Sha_conj
1.0


Again we emphasize that we do not even know that
the conjectural order computed above is a rational number.

It seems almost a miracle that $L^{(4)}(E,1) = 214.65\ldots$,
$\Omega_E = 2.97\ldots$, and $\Reg(E) = 1.50\ldots$ have
anything to do with each other, but indeed they do:
\noindent{}

>>> L/Integer(24), Integer(2)*Om*Reg###line 1004:_sage_    : L/24, 2*Om*Reg
(8.9438473959, 8.9438473959)
"""

def example_6():	r"""
>>> E = EllipticCurve('446d1'); p = Integer(5); E###line 1096:_sage_    : E = EllipticCurve('446d1'); p = 5; E
Elliptic Curve defined by y^2 + x*y  = x^3 - x^2 - 4*x + 4
over Rational Field


\noindent{}Next we verify that the rank is $2$, that $p$
is a good ordinary prime, and that there are $10$ points
on $E$ modulo $p$ (so $E$ is {\em ananomolous at $p$}, i.e.,
$p\mid \#E(\FF_p)$).

>>> E.rank()###line 1107:_sage_    : E.rank()
2
>>> E.is_ordinary(p)###line 1109:_sage_    : E.is_ordinary(p)
True
>>> E.Np(p)###line 1111:_sage_    : E.Np(p)
10


\noindent{}Next we compute the $p$-adic $L$-series of $E$ at $p$.
We add $O(T^7)$ so that the displayed series doesn't take several lines.
% note -- make the precision bigger than 4 at some point!

>>> Lp  = E.padic_lseries(p)###line 1120:_sage_    : Lp  = E.padic_lseries(p)
>>> LpT = Lp.series(Integer(4))###line 1121:_sage_    : LpT = Lp.series(4)
>>> LpT = LpT.add_bigoh(Integer(7)); LpT###line 1122:_sage_    : LpT = LpT.add_bigoh(7); LpT
(5 + 5^2 + O(5^3))*T^2 + (2*5 + 3*5^2 + O(5^3))*T^3
          + (4*5^2 + O(5^3))*T^4 + (4*5 + O(5^2))*T^5
          + (1 + 2*5 + O(5^3))*T^6 + O(T^7)


\noindent{}We compute the $p$-adic modular form $E_2$ evaluated
on our elliptic curve with differential $\omega$ to precision $O(5^{8})$.
This is the key difficult input to the computation of the $p$-adic
regulator $\Reg_p(E)$.

>>> E.padic_E2(p, prec=Integer(8))###line 1134:_sage_    : E.padic_E2(p, prec=8)
3*5 + 4*5^2 + 5^3 + 5^4 + 5^5 + 2*5^6 + 4*5^7 + O(5^8)


\noindent{}We compute the normalized $p$-adic regulator,
normalized to the choice of $1+p$ as a topological
generator of $1+p\,\mathbb{Z}_p$.
% note -- this normalization will be automatic at some point soon.

>>> Regp = E.padic_regulator(p, Integer(10))###line 1144:_sage_    : Regp = E.padic_regulator(p, 10)
>>> R = Regp.parent()###line 1145:_sage_    : R = Regp.parent()
>>> kg = log(R(Integer(1)+p))###line 1146:_sage_    : kg = log(R(1+p))
>>> reg = Regp * p**Integer(2) / log(R(Integer(1)+p))**Integer(2)###line 1147:_sage_    : reg = Regp * p^2 / log(R(1+p))^2
>>> reg*kg**Integer(2)###line 1148:_sage_    : reg*kg^2
2*5 + 2*5^2 + 5^4 + 4*5^5 + 2*5^7 + O(5^8)


\noindent{}We compute the Tamagawa numbers and torsion subgroup.

>>> E.tamagawa_numbers()###line 1155:_sage_    : E.tamagawa_numbers()
[2, 1]
>>> E.torsion_order()###line 1157:_sage_    : E.torsion_order()
1


\noindent{}We compute $\cL_p^*(E,0)$, which is the leading term of
the $p$-adic $L$-function.  It is not a
unit, so we call the prime $p$ an {\em irregular} prime.

>>> Lpstar = LpT[Integer(2)]; Lpstar###line 1166:_sage_    : Lpstar = LpT[2]; Lpstar
5 + 5^2 + O(5^3)


\noindent{}Finally, putting everything together
we compute the conjectural $p$-adic order of $\#\Sha(E/\QQ)$.
In particular, we see that conjecturally
$\#\Sha(E/\QQ)(5)$ is trivial.

>>> eps = (Integer(1)-Integer(1)/Lp.alpha(Integer(20)))**Integer(2)###line 1176:_sage_    : eps = (1-1/Lp.alpha(20))^2
>>> Lpstar / (eps*reg*(Integer(2)*Integer(1))) * (Integer(1))**Integer(2)###line 1177:_sage_    : Lpstar / (eps*reg*(2*1)) * (1)^2
1 + O(5^2)
"""


if __name__ ==  '__main__':
    import doctest, sys
    s = doctest.testmod(sys.modules[__name__],
                   optionflags=doctest.NORMALIZE_WHITESPACE
                              |doctest.ELLIPSIS,
                   verbose=False,
                   globs=globals())
    quit_sage(verbose=False)
    sys.exit(s[0])